from .operation_type import OperationType


class SqlResultPaginationPayload(object):
    def __init__(self) -> None:
        self._name = OperationType.SQL_RESULT_PAGINATION.value
        self._query = """
            query sqlResultPagination($where: SqlResultPaginationWhereInput!) {
                sqlResultPagination(where: $where) {
                    result
                }
            }
        """

    def build(self, sql_query_id: str, limit: int, offset: int) -> dict:
        return {
            "operationName": self._name,
            "query": self._query,
            "variables": {
                "where": {
                    "id": sql_query_id,
                    "limit": limit,
                    "offset": offset,
                    "source": "NOTEBOOK",
                }
            },
        }


import datetime
import os
import pandas as pd
import pytz
from cannerflow.utils import *
from numpy import ndarray
from pandas import DataFrame, Timestamp

PARQUET_FILE = os.path.join(os.path.dirname(__file__), 'simpletest.parquet')

ans = {
    'cbool': True,
    'ctinyint': pow(2, 7) - 1,
    'csmallint': pow(2, 15) - 1,
    'cint': pow(2, 31) - 1,
    'cbigint': pow(2, 62),
    'creal': 10.3,
    'cdouble': 10.3,
    'cdecimal': 100.34,
    'cvarchar': 'abcdefghijk',
    'cchar': 'abcdefghijk',
    'cvarbinary': b'abcdefghijk',
    'cjson': '{"a":1,"b":2}',
    'cdate': Timestamp("2001-08-22"),
    'ctimestamp': Timestamp(
        datetime.datetime.strptime("2001-08-22 03:04:05.321", "%Y-%m-%d %H:%M:%S.%f").astimezone(pytz.utc)).tz_localize(
        None),
    'ctimestampwithtz': '2001-08-22 03:04:05.321 America/Los_Angeles',
    'carray': [1, 2, 3],
    'cmap': {b'foo': b'foo1', b'boo': b'boo1'},
    'cipaddress': '10.0.0.1',
    'cuuid': '12151fd2-7586-11e9-8f9e-2a86e4085a59',
    'crow.x': 1,
    'crow.y': 2.0
}


def test_parquet_to_list():
    df = pd.read_parquet(PARQUET_FILE, engine="fastparquet")
    columns = list(map(lambda column: {'name': column}, df.columns))
    data: list = data_factory(data_format="list", columns=columns, data=df.to_numpy().tolist())
    assert len(ans) == len(data[0]), "the number of elements expected: " + str(len(ans)) + " but actual: " + str(
        len(data[0]))
    for i in range(len(data[0])):
        actual = data[1][i]
        # TODO: ignore the precision issue for now, we would fix it in the future.
        # Expected 10.3 but actual is 10.300000190734863.
        if data[0][i] == 'creal':
             actual = round(actual, 2)
        expected = ans[data[0][i]]

        assert actual == expected, "data[" + str(i) + "] expected: " + str(
            ans[data[0][i]]) + " but actual: " + str(data[1][i])


def test_parquet_to_df():
    df = pd.read_parquet(PARQUET_FILE, engine="fastparquet")
    columns = list(map(lambda column: {'name': column}, df.columns))
    data: DataFrame = data_factory(data_format="df", columns=columns, data=df.to_numpy().tolist())
    assert len(ans) == len(data.columns), "the number of elements expected: " + str(len(ans)) + " but actual: " + str(
        len(data[0]))
    for i in range(len(data.columns)):
        actual = data[data.columns[i]][0]
        # same as `test_parquet_to_list`
        if data.columns[i] == 'creal':
            actual = round(actual, 2)
        expected = ans[data.columns[i]]

        assert actual == expected, "data[" + str(i) + "] expected: " + str(
            ans[data.columns[i]]) + " but actual: " + str(data[data.columns[i]].values[0])


def test_parquet_to_np():
    df = pd.read_parquet(PARQUET_FILE, engine="fastparquet")
    columns = list(map(lambda column: {'name': column}, df.columns))
    data: ndarray = data_factory(data_format="np", columns=columns, data=df.to_numpy().tolist())
    assert len(ans) == len(df.columns), "the number of elements expected: " + str(len(ans)) + " but actual: " + str(
        len(df.columns))
    for i in range(len(df.columns)):
        actual = data[0][i]
        # same as `test_parquet_to_list`
        if df.columns[i] == 'creal':
            actual = round(actual, 2)
        expected = ans[df.columns[i]]

        assert actual == expected, "data[" + str(i) + "] expected: " + str(
            ans[df.columns[i]]) + " but actual: " + str(data[0][i])



from .operation_type import OperationType


class DeleteStatementPayload(object):
    def __init__(self) -> None:
        self._name = OperationType.SQL_RESULT_PAGINATION.value
        self._query = """
            mutation deleteStatement($where: StatementWhereUniqueInput!) {
                deleteStatement(where: $where) {
                    id
                }
            }
        """

    def build(self, statement_id: str) -> dict:
        return {
            "operationName": self._name,
            "query": self._query,
            "variables": {"where": {"id": statement_id}},
        }


from typing import Any, Union, Optional

from pandas.core.frame import DataFrame
from cannerflow.logging import *
from cannerflow.utils import *
from cannerflow.payloads import *
from cannerflow.exceptions import handle_exception
from cannerflow.request import CannerRequest
from cannerflow.dto import SqlQueryResult, SqlQueryStatus
from cannerflow.adapters.query_reader import ParquetQueryReader, ApiFetchQueryReader

# from cannerflow.query_reader import

__all__ = ["Query"]


class Query(object):
    def __init__(
        self,
        request: CannerRequest,
        workspace_id: str,
        sql: str,
        cache_refresh: bool,
        cache_ttl: int,
        data_format: str,
        page_limit: int = 5000,
        fetch_by: str = "storage",
    ):
        self._workspace_id = workspace_id
        self._request = request
        self._sql = sql
        self._page_limit = page_limit
        self._fetch_by = fetch_by
        self._columns: Optional[Union[dict, list]] = None
        self.data_format = data_format

        sql_query = self.___create_sql_query(
            sql,
            workspace_id,
            cache_refresh,
            cache_ttl,
        )
        self.__update_query_info(sql_query)

    @property
    def id(self) -> str:
        return self._id

    @property
    def row_count(self) -> Optional[int]:
        """
        row count will shows the correct value until `status` shows `FINISHED`.
        If `status` not be `FINISHED`, the row count return `None`.
        """
        return None if self._status != SqlQueryStatus.FINISHED else self._row_count

    @property
    def status(self) -> SqlQueryStatus:
        return self._status

    @property
    def statement_id(self) -> Optional[str]:
        return self._statement_id

    @property
    def columns(self) -> Union[list, dict, None]:
        return self._columns

    def __ensure_query_finished(self):
        if self._status != SqlQueryStatus.FINISHED:
            self.wait_for_finish()

    def __update_query_info(self, sql_query: SqlQueryResult):
        self._id = sql_query.id
        self._status = SqlQueryStatus(sql_query.status)
        self._error = sql_query.error
        self._row_count = sql_query.row_count
        self._columns = sql_query.columns
        self._statement_id = sql_query.statementId

    def __iter__(self):
        row, offest, page = 0, 0, 1
        # ensure query status finished
        self.__ensure_query_finished()
        # prepare storage urls by creating parquet reader and
        parquet_reader = ParquetQueryReader(self._id, self._workspace_id, self._request)
        # fetch data of first page
        page_result = parquet_reader.read(self._page_limit, offest)

        while (page - 1) * self._page_limit + row < self._row_count:
            if row == len(page_result.data):
                offest = self._page_limit * page
                # fetch data of next page and reset index row = 0
                page_result = parquet_reader.read(self._page_limit, offest)
                row, page = 0, page + 1
            elif row < len(page_result.data):
                formatted_data = self.__transfer_format(
                    self.columns, [page_result.data[row]], self.data_format
                )
                yield formatted_data
                row += 1
            else:
                break

    @handle_exception
    def ___create_sql_query(
        self, sql: str, workspace_id: str, cache_refresh: bool, cache_ttl: int
    ) -> SqlQueryResult:
        payload = CreateSqlQueryPayload().build(
            sql,
            workspace_id,
            cache_refresh,
            cache_ttl,
        )
        resp = self._request.graphql_exec(payload)
        result = resp.get("createSqlQuery")
        result = SqlQueryResult(**result)
        if result.status == SqlQueryStatus.FAILED:
            print(result)
            raise Exception("Execute sql failed")
        return result

    @handle_exception
    def __get_sql_query(self) -> SqlQueryResult:
        response = self._request.graphql_exec(SqlQueryPayload().build(self._id))
        result = response.get("sqlQuery")
        return SqlQueryResult(**result)

    @handle_exception
    def delete_statement(self):
        if self._statement_id is None:
            return
        self._request.graphql_exec(DeleteStatementPayload().build(self._statement_id))
        # update query infomation
        sql_query = self.__get_sql_query()
        self.__update_query_info(sql_query)

    @handle_exception
    def get_sql_result(self, limit, offset) -> Union[Any, list]:
        if self._fetch_by == "storage":
            return self.__read_sql_result_by_storage(limit, offset)
        else:
            return self.__read_sql_result_by_restful(limit, offset)

    def __read_sql_result_by_storage(self, limit, offset) -> list:
        parquet_reader = ParquetQueryReader(self._id, self._workspace_id, self._request)
        output = parquet_reader.read(limit, offset)
        # Fetch metadata again for getting correct row count after query finished
        sql_query = self.__get_sql_query()
        self.__update_query_info(sql_query)
        # if parquet column has value, then used parquet format columns field, otherwise keep original column field (because no data, so keep original column field information)
        if output.columns:
            self._columns = output.columns
        return output.data

    def __read_sql_result_by_restful(self, limit, offset):
        output_data = []
        api_fetch_reader = ApiFetchQueryReader(
            self._id, self._page_limit, self._request
        )
        output_data = api_fetch_reader.read(limit, offset)
        # Fetch metadata again for getting correct row count after query finished
        sql_query = self.__get_sql_query()
        self.__update_query_info(sql_query)
        return output_data

    @handle_exception
    def wait_for_finish(self, timeout=120, period=1):
        def check_status_and_update_info():
            if self._status != SqlQueryStatus.FINISHED:
                sql_query = self.__get_sql_query()
                self.__update_query_info(sql_query)
                return False
            return True

        wait_until(check_status_and_update_info, timeout, period)

    @handle_exception
    def __transfer_format(
        self, columns: Union[list, Any, None], data: Union[list, Any], format: str
    ) -> Union[Any, DataFrame]:
        try:
            return data_factory(data_format=format, columns=columns, data=data)
        except Exception:
            raise Exception(
                "Cannot get data correctly, please run query.wait_for_finish(timeout=seconds,period=seconds) first"
            )

    @handle_exception
    def get_all(self):
        # ensure query finished and could get full row count
        self.__ensure_query_finished()
        query_data = self.get_sql_result(self._row_count, 0)
        return self.__transfer_format(self.columns, query_data, self.data_format)

    @handle_exception
    def get_first(self, limit=1):
        query_data = self.get_sql_result(limit, 0)
        return self.__transfer_format(self.columns, query_data, self.data_format)

    @handle_exception
    def get_last(self, limit=1):
        # ensure query finished and could get full row count
        self.__ensure_query_finished()
        offset = self._row_count - limit
        query_data = self.get_sql_result(limit, offset)
        return self.__transfer_format(self.columns, query_data, self.data_format)

    @handle_exception
    def get(self, limit, offset):
        query_data = self.get_sql_result(limit, offset)
        return self.__transfer_format(self.columns, query_data, self.data_format)



from typing import Optional
from cannerflow.request import CannerRequest
from cannerflow.saved_query import SavedQuery
from cannerflow.query import Query
from cannerflow.file import File

__all__ = ["Workspace"]


class Workspace(object):
    def __init__(
        self,
        endpoint: str,
        workspace_id: str,
        replaceLocalhostString: Optional[str] = None,
        headers: Optional[dict] = None,
    ):
        self.replaceLocalhostString = replaceLocalhostString
        self.endpoint = self._replace_localhost(endpoint)
        self.headers = headers
        self.workspace_id = workspace_id
        request = CannerRequest(headers=headers, endpoint=self.endpoint)
        self.request = request
        self.saved_query = SavedQuery(request=request, workspace_id=workspace_id)
        self.file = File(
            request=request,
            workspace_id=workspace_id,
            replaceLocalhostString=replaceLocalhostString,
        )

    def gen_query(
        self,
        sql: str,
        cache_refresh: bool,
        cache_ttl: int,
        data_format: str,
        fetch_by: str,
    ):
        return Query(
            request=self.request,
            workspace_id=self.workspace_id,
            sql=sql,
            cache_refresh=cache_refresh,
            cache_ttl=cache_ttl,
            data_format=data_format,
            fetch_by=fetch_by,
        )

    def _replace_localhost(self, url: str):
        if self.replaceLocalhostString is not None:
            return url.replace("localhost", self.replaceLocalhostString).replace(
                "127.0.0.1", self.replaceLocalhostString
            )
        return url


import json
import pandas
from io import BytesIO

__all__ = ["JsonWrapper"]


class JsonWrapper(object):
    def __init__(self, content, encoding="utf-8"):
        self.content = content
        self.encoding = encoding

    def to_json(self):
        self.content_text = self.content.decode(self.encoding)
        return json.loads(self.content_text)

    def to_pandas(self, *args, **kwargs):
        return pandas.read_json(path_or_buf=BytesIO(self.content), *args, **kwargs)



import sys
import requests
from typing import Optional
from http import HTTPStatus
from requests.models import Response
from cannerflow.exceptions import CannerflowError, ConnectionError
from cannerflow.logging import get_logger

__all__ = ["CannerRequest"]
logger = get_logger("CannerRequest")


class CannerRequest(object):
    def __init__(self, endpoint: str, headers: Optional[dict] = None):
        self._endpoint = endpoint
        self._headers = headers

    def _parse_graphql(self, resp: Response):
        # handle non 200 error (not from cannerflow backend graphql e.g: some middlewares)
        if resp.status_code != HTTPStatus.OK:
            # Check the graphql response error format
            if resp.text and "code" in resp.text:
                error = resp.json()
                raise CannerflowError(
                    error_code=error["code"], message=error["message"]
                )
            raise ConnectionError(message=resp.text)
        else:
            result = resp.json()
            # handle graphql error (status code is 200, but check error from errors)
            if "errors" in result:
                error = result["errors"][0]
                raise CannerflowError(
                    error_code=error["extensions"]["code"], message=error["message"]
                )
            data = result.get("data")
            return data

    def _parse_restful(self, resp: Response):
        if resp.status_code != HTTPStatus.OK:
            # Check the restful response error format
            if resp.text and "code" in resp.text:
                error = resp.json()
                raise CannerflowError(
                    error_code=error["code"], message=error["message"]
                )
            raise ConnectionError(message=resp.text)
        # If status code is 200 or not 200
        data = resp.json()
        return data

    def graphql_exec(self, payload: dict):
        """
        The graphql request method
        """
        graphql_url = f"{self._endpoint}/graphql"
        resp: Response = requests.post(
            url=graphql_url, json=payload, headers=self._headers
        )
        return self._parse_graphql(resp)

    def get(self, path: str):
        """
        The restful HTTP Get request method
        """
        rest_url = f"{self._endpoint}/{path}"
        resp: Response = requests.get(url=rest_url, headers=self._headers)
        return self._parse_restful(resp)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import json
import csv
from io import StringIO
from typing import Optional
import pandas as pd
import cannerflow.workspace
import cannerflow.exceptions
import cannerflow.utils
import nest_asyncio

nest_asyncio.apply()

__all__ = ["bootstrap", "Client"]


def bootstrap(*args, **kwargs):
    return Client(*args, **kwargs)


class Client(object):
    def __init__(
        self,
        endpoint: str,
        workspace_id: str,
        replaceLocalhostString: Optional[str] = None,
        headers: Optional[dict] = None,
        token: Optional[str] = None,
    ):
        endpoint = endpoint.rstrip("/")
        if token != None:
            if not headers:
                headers = {"Authorization": f"Token {token}"}
            else:
                headers["Authorization"] = f"Token {token}"
        # Create workspace
        self.workspace = cannerflow.workspace.Workspace(
            endpoint=endpoint,
            headers=headers,
            workspace_id=workspace_id,
            replaceLocalhostString=replaceLocalhostString,
        )

    # SQL
    def use_saved_query(
        self, title, cache_refresh=False, cache_ttl=86400, data_format="list"
    ):
        query = self.workspace.saved_query.get(title)
        return self.gen_query(
            sql=query["sql"],
            cache_refresh=cache_refresh,
            cache_ttl=cache_ttl,
            data_format=data_format,
        )

    def list_saved_query(self):
        return self.workspace.saved_query.list_title()

    def gen_query(
        self,
        sql,
        cache_refresh=False,
        cache_ttl=86400,
        data_format="list",
        fetch_by="storage",
    ):
        query = self.workspace.gen_query(
            sql,
            cache_refresh=cache_refresh,
            cache_ttl=cache_ttl,
            data_format=data_format,
            fetch_by=fetch_by,
        )
        return query

    # files
    def list_file(self):
        return self.workspace.file.list_absolute_path()

    def get_binary(self, absolute_path):
        return self.workspace.file.get_content(absolute_path)

    def put_binary(self, absolute_path, data):
        return self.workspace.file.put(absolute_path, data)

    # wrappers
    def _get_csv_wrapper(self, absolute_path, encoding="utf-8"):
        return self.workspace.file.get_csv_wrapper(absolute_path, encoding=encoding)

    def _get_json_wrapper(self, absolute_path, encoding="utf-8"):
        return self.workspace.file.get_json_wrapper(absolute_path, encoding=encoding)

    def _get_image_wrapper(self, absolute_path):
        return self.workspace.file.get_image_wrapper(absolute_path)

    # csv
    def get_csv(self, absolute_path, encoding="utf-8"):
        return self._get_csv_wrapper(absolute_path, encoding).to_list()

    def get_pandas_csv(self, absolute_path, encoding="utf-8", *args, **kwargs):
        return self._get_csv_wrapper(absolute_path, encoding).to_pandas(*args, **kwargs)

    def put_csv(self, absolute_path, data, encoding="utf-8", *args, **kwargs):
        if isinstance(data, pd.DataFrame):
            return self.put_pandas_csv(absolute_path, data, encoding, *args, **kwargs)
        return self.put_csv_str(absolute_path, data, encoding, *args, **kwargs)

    def put_csv_str(self, absolute_path, data, encoding="utf-8", *args, **kwargs):
        str_io = StringIO()
        csv_writer = csv.writer(str_io, *args, **kwargs)
        csv_writer.writerows(data)
        value = str_io.getvalue().encode(encoding)
        return self.put_binary(absolute_path=absolute_path, data=value)

    def put_pandas_csv(self, absolute_path, data, encoding="utf-8"):
        data = data.to_csv(index=False).encode(encoding)
        return self.put_binary(absolute_path=absolute_path, data=data)

    # json
    def get_json(self, absolute_path, encoding="utf-8"):
        return self._get_json_wrapper(absolute_path, encoding).to_json()

    def get_pandas_json(self, absolute_path, encoding="utf-8", *args, **kwargs):
        return self._get_json_wrapper(absolute_path, encoding).to_pandas(
            *args, **kwargs
        )

    def put_json(self, absolute_path, data, encoding="utf-8"):
        if isinstance(data, pd.DataFrame):
            return self.put_pandas_json(absolute_path, data, encoding)
        else:
            return self.put_json_str(absolute_path, data, encoding)

    def put_json_str(self, absolute_path, data, encoding="utf-8"):
        data = json.dumps(data).encode(encoding)
        return self.put_binary(absolute_path=absolute_path, data=data)

    def put_pandas_json(self, absolute_path, data, encoding="utf-8"):
        data = data.to_json()
        self.put_json_str(absolute_path=absolute_path, data=data, encoding=encoding)

    # image
    def get_pil_image(self, absolute_path):
        return self._get_image_wrapper(absolute_path).to_pil()

    def get_np_image(self, absolute_path):
        return self._get_image_wrapper(absolute_path).to_np()



from cannerflow.utils import *
import pandas as pd
import numpy as np

data = [
    [
        "FINISHED",
        "2019-11-20 07:32:21.363",
        209,
        None,
        None,
        "select * from cannerflow.1.sq",
        "f648ac0e-7a87-409f-bed0-27a1fb9e9927",
        None,
        None,
        "20191120_070156_00003_q7byd",
        "default/query/data/20191120_070156_00003_q7byd/data",
        "BASIC_USER",
        0,
        0,
        4,
        True
    ],
    [
        "FINISHED",
        "2019-11-20 07:30:53.979",
        1301,
        None,
        None,
        "select * from cannerflow.1.sq",
        "f648ac0e-7a87-409f-bed0-27a1fb9e9927",
        None,
        None,
        "20191115_023945_00063_8553e",
        "default/query/data/20191115_023945_00063_8553e/data",
        "BASIC_USER",
        0,
        866,
        0,
        True
    ]
]
columns = [
    {
      "name": "status",
      "type": "varchar",
      "typeSignature": {
        "rawType": "varchar",
        "typeArguments": [],
        "literalArguments": [],
        "arguments": [
          {
            "kind": "LONG_LITERAL",
            "value": 2147483647
          }
        ]
      }
    },
  {
    "name": "started_at",
    "type": "timestamp",
    "typeSignature": {
      "rawType": "timestamp",
      "typeArguments": [],
      "literalArguments": [],
      "arguments": []
    }
  },
  {
    "name": "duration",
    "type": "double",
    "typeSignature": {
      "rawType": "double",
      "typeArguments": [],
      "literalArguments": [],
      "arguments": []
    }
  },
  {
    "name": "columns",
    "type": "json",
    "typeSignature": {
      "rawType": "json",
      "typeArguments": [],
      "literalArguments": [],
      "arguments": []
    }
  },
  {
    "name": "data",
    "type": "json",
    "typeSignature": {
      "rawType": "json",
      "typeArguments": [],
      "literalArguments": [],
      "arguments": []
    }
  },
  {
    "name": "sql",
    "type": "varchar",
    "typeSignature": {
      "rawType": "varchar",
      "typeArguments": [],
      "literalArguments": [],
      "arguments": [
        {
          "kind": "LONG_LITERAL",
          "value": 2147483647
        }
      ]
    }
  },
  {
    "name": "workspace_id",
    "type": "varchar",
    "typeSignature": {
      "rawType": "varchar",
      "typeArguments": [],
      "literalArguments": [],
      "arguments": [
        {
          "kind": "LONG_LITERAL",
          "value": 2147483647
        }
      ]
    }
  },
  {
    "name": "error",
    "type": "json",
    "typeSignature": {
      "rawType": "json",
      "typeArguments": [],
      "literalArguments": [],
      "arguments": []
    }
  },
  {
    "name": "data_link",
    "type": "varchar",
    "typeSignature": {
      "rawType": "varchar",
      "typeArguments": [],
      "literalArguments": [],
      "arguments": [
        {
          "kind": "LONG_LITERAL",
          "value": 2147483647
        }
      ]
    }
  },
  {
    "name": "statement_id",
    "type": "varchar",
    "typeSignature": {
      "rawType": "varchar",
      "typeArguments": [],
      "literalArguments": [],
      "arguments": [
        {
          "kind": "LONG_LITERAL",
          "value": 2147483647
        }
      ]
    }
  },
  {
    "name": "location",
    "type": "varchar",
    "typeSignature": {
      "rawType": "varchar",
      "typeArguments": [],
      "literalArguments": [],
      "arguments": [
        {
          "kind": "LONG_LITERAL",
          "value": 2147483647
        }
      ]
    }
  },
  {
    "name": "user_type",
    "type": "varchar",
    "typeSignature": {
      "rawType": "varchar",
      "typeArguments": [],
      "literalArguments": [],
      "arguments": [
        {
          "kind": "LONG_LITERAL",
          "value": 2147483647
        }
      ]
    }
  },
  {
    "name": "in_bound",
    "type": "integer",
    "typeSignature": {
      "rawType": "integer",
      "typeArguments": [],
      "literalArguments": [],
      "arguments": []
    }
  },
  {
    "name": "out_bound",
    "type": "integer",
    "typeSignature": {
      "rawType": "integer",
      "typeArguments": [],
      "literalArguments": [],
      "arguments": []
    }
  },
  {
    "name": "row_count",
    "type": "integer",
    "typeSignature": {
      "rawType": "integer",
      "typeArguments": [],
      "literalArguments": [],
      "arguments": []
    }
  },
  {
    "name": "from_cache",
    "type": "boolean",
    "typeSignature": {
      "rawType": "boolean",
      "typeArguments": [],
      "literalArguments": [],
      "arguments": []
    }
  }
]

def test_data_factory_list():
    data_list = data_factory(data_format='list', columns=columns, data=data)
    assert data_list[0] == list(map(lambda column: column['name'], columns)), "First row should be column list"
    assert data_list[1:] == data, "Last row should be data list"

def test_data_factory_df():
    data_list = data_factory(data_format='df', columns=columns, data=data)
    assert isinstance(data_list, pd.DataFrame), "Should be a data frame"
def test_data_factory_np():
    data_list = data_factory(data_format='np', columns=columns, data=data)
    assert isinstance(data_list, np.ndarray), "Should be a np array"


if __name__ == '__main__':
    test_data_factory_list()
    test_data_factory_df()
    test_data_factory_np()
    print("Tests passed")


import time
from cannerflow.request import CannerRequest
from cannerflow.payloads import SqlResultPaginationPayload


class ApiFetchQueryReader(object):
    def __init__(self, query_id: str, page_limit: int, request: CannerRequest) -> None:
        self._query_id = query_id
        self._page_limit = page_limit
        self._request = request
        self._payload = SqlResultPaginationPayload()

    def read(self, limit: int, offset: int) -> list:
        # If the source limit largest fetch
        output_data = []

        if limit > self._page_limit:
            page_number = 0
            while page_number * self._page_limit < limit:
                page_offest = page_number * self._page_limit
                response = self._request.graphql_exec(
                    self._payload.build(self._query_id, self._page_limit, page_offest)
                )
                result = response.get("sqlResultPagination")
                output_data.extend(result["result"])
                page_number += 1
                # sleep to prevent server refused connection as retry too many times
                time.sleep(1)
        else:
            response = self._request.graphql_exec(
                self._payload.build(self._query_id, limit, offset)
            )
            result = response.get("sqlResultPagination")
            output_data = result["result"]
        return output_data


from enum import Enum


class OperationType(Enum):
    CREATE_SQL_QUERY = "createSqlQuery"
    SQL_QUERY = "sqlQuery"
    SQL_RESULT_PAGINATION = "sqlResultPagination"
    DELETE_STATEMENT = "deleteStatement"



__all__ = ["SavedQuery"]

from cannerflow.request import CannerRequest


class SavedQuery(object):
    def __init__(self, request: CannerRequest, workspace_id: str):
        self.workspace_id = workspace_id
        self.request = request

    def getPayload(self):
        return {
            "operationName": "sqls",
            "query": """
                query sqls($where: SqlWhereInput!) {
                    sqls(where: $where) {
                        id
                        sql
                        title
                        description
                        workspaceId
                    }
                }
            """,
            "variables": {"where": {"workspaceId": self.workspace_id}},
        }

    def fetch(self):
        return self.request.graphql_exec(self.getPayload()).get("sqls")

    def list_title(self):
        data = self.fetch()
        return list(map(lambda x: x["title"], data))

    def get(self, title):
        queries = self.fetch()
        query = next(q for q in queries if q.get("title") == title)
        return query



import os
from cannerflow.file import *

CURRENT_PATH = os.path.dirname(__file__)

NORMAL_CSV = os.path.join(CURRENT_PATH, "normal.csv")
MBCS_CSV = os.path.join(CURRENT_PATH, "mbcs.csv")


def test_csv_wrapper_utf8_bom():
    with open(NORMAL_CSV, "rb") as f:
        content = f.read()
    csv_wrapper = CsvWrapper(content=content, encoding="utf-8-sig")
    csv = csv_wrapper.to_list()
    assert len(csv) > 0, "Should parse csv correctly"
    assert csv[0] == ["id", "name"], "first row should be column name"


def test_csv_wrapper_mbcs():
    with open(MBCS_CSV, "rb") as f:
        content = f.read()
    csv_wrapper = CsvWrapper(content=content, encoding="cp950")
    csv = csv_wrapper.to_pandas()
    assert len(csv) > 0, "Should parse csv correctly"


if __name__ == "__main__":
    test_csv_wrapper_utf8_bom()
    test_csv_wrapper_mbcs()
    print("Tests passed")


from enum import Enum
from typing import Optional, Union
from dataclasses import dataclass


class SqlQueryStatus(Enum):
    QUEUED = "QUEUED"
    # Query has been accepted and is awaiting execution.
    WAITING_FOR_RESOURCES = "WAITING_FOR_RESOURCES"
    # Query is waiting for the required resources (beta).
    DISPATCHING = "DISPATCHING"
    # Query is being dispatched to a coordinator.
    PLANNING = "PLANNING"
    # Query is being planned.
    STARTING = "STARTING"
    # Query execution is being started.
    RUNNING = "RUNNING"
    # Query has at least one running task.
    FINISHING = "FINISHING"
    # Query is finishing (e.g. commit for autocommit queries)
    FINISHED = "FINISHED"
    # Query has finished executing and all output has been consumed.
    EXPIRED = "EXPIRED"
    # Query has expired.
    FAILED = "FAILED"
    # Query execution failed.
    CANCELED = "CANCELED"
    ABORTED = "ABORTED"
    PLANNED = "PLANNED"


@dataclass(frozen=True)
class SqlQueryResult(object):
    id: str
    status: str
    error: dict
    statementId: str
    rowCount: int = 0
    columns: Optional[Union[dict, list]] = None
    location: Optional[str] = None

    # just for rename use
    @property
    def row_count(self) -> int:
        return self.rowCount



from dataclasses import dataclass
import pandas as pd
import requests
from io import BytesIO
from typing import List, Any
from cannerflow.request import CannerRequest
from cannerflow.logging import *


@dataclass
class ParquetQueryResult(object):
    columns: List[Any]
    data: Any


class ParquetQueryReader(object):
    def __init__(
        self,
        query_id: str,
        workspace_id: str,
        request: CannerRequest,
    ) -> None:

        self._query_id = query_id
        self._workspace_id = workspace_id
        self._request = request
        self._engine = "fastparquet"
        self._storage_urls = self.__retrieve_storage_urls()

    def read(self, limit: int, offset: int) -> ParquetQueryResult:
        # The columns is from parquet read
        query_columns = []
        self.__check_nested_collection_and_warning()
        if len(self._storage_urls) == 0:
            df = pd.DataFrame({})
        else:
            dfs = []
            count = 0
            for url in self._storage_urls:
                df = self.__get_pd_by_parquet_file(url, self._engine)
                count += df.shape[0]
                dfs.append(df)
                # if we get enough data, stop getting the next one
                if count >= limit + offset:
                    df = pd.concat(dfs)
                    break
            query_columns = list(map(lambda column: {"name": column}, list(df.columns)))
        result = df.iloc[offset : limit + offset, :].to_numpy().tolist()
        # if query_columns is [] means, not storage url (no data)
        return ParquetQueryResult(query_columns, result)

    def __retrieve_storage_urls(self) -> List[str]:
        path_url = f"api/v1/query/{self._query_id}/result/signedUrls?workspaceId={self._workspace_id}"
        return self._request.get(path_url).get("signedUrls")

    def __get_pd_by_parquet_file(self, url, engine):
        r = requests.get(url)
        return pd.read_parquet(BytesIO(r.content), engine=engine)

    def __check_nested_collection_and_warning(self):
        result = self._request.get(
            f"api/v1/query/{self._query_id}?workspaceId={self._workspace_id}"
        )
        columns = result["columns"]
        logger = get_logger("Query", log_level=logging.WARNING)
        for column in columns:
            if self.__is_collection(column["typeSignature"]["rawType"]):
                for eleColumn in column["typeSignature"]["arguments"]:
                    if eleColumn["kind"] == "TYPE":
                        if self.__is_collection(eleColumn["value"]["rawType"]):
                            logger.warning(
                                f"Nested Collection isn't supported in fetchBy-storage-mode. Column %s: %s "
                                f"would be None.",
                                column["name"],
                                column["type"],
                            )
                    elif eleColumn["kind"] == "NAMED_TYPE":
                        if self.__is_collection(
                            eleColumn["value"]["typeSignature"]["rawType"]
                        ):
                            logger.warning(
                                f"Nested Collection isn't supported in fetchBy-storage-mode. Column %s: %s "
                                f"would be None.",
                                column["name"],
                                column["type"],
                            )

    def __is_collection(self, type):
        if ["array", "map", "row"].__contains__(type):
            return True
        return False



import csv
import os
from re import I

import json_lines
import numpy as np
import pandas as pd
import pytest
from PIL.PngImagePlugin import PngImageFile

import cannerflow
from cannerflow.dto import SqlQueryStatus

ENDPOINT = os.getenv("ENDPOINT") or "http://localhost:3000"

WORKSPACE_ID = os.getenv("WORKSPACE_ID")
if WORKSPACE_ID is None:
    raise EnvironmentError("No workspace id")

JUPYTER_SECRET = os.getenv("CANNERFLOW_JUPYTER_SECRET") or "jupyter-secret"
TOKEN = os.getenv("CANNERFLOW_PERSONAL_ACCESS_TOKEN")
if WORKSPACE_ID is None:
    raise EnvironmentError("No personal access token")

CURRENT_PATH = os.path.dirname(__file__)
CSV_FILE = os.path.join(CURRENT_PATH, "test.csv")
JSON_FILE = os.path.join(CURRENT_PATH, "orders.json")
IMAGE_FILE = os.path.join(CURRENT_PATH, "lena256.png")
NORMAL_CSV = os.path.join(CURRENT_PATH, "normal.csv")

print(
    {
        "endpoint": ENDPOINT,
        "workspaceId": WORKSPACE_ID,
        "csvFile": CSV_FILE,
        "secret": JUPYTER_SECRET,
        "token": TOKEN,
    }
)

# case one, secret-based client
secret_client = cannerflow.client.bootstrap(
    endpoint=ENDPOINT,
    workspace_id=WORKSPACE_ID,
    headers={
        "X-CANNERFLOW-SECRET": JUPYTER_SECRET,
        "X-CANNERFLOW-WORKSPACE-ID": WORKSPACE_ID,
    },
)

# case two, token-based client
token_client = cannerflow.client.bootstrap(
    endpoint=ENDPOINT,
    workspace_id=WORKSPACE_ID,
    token=TOKEN,
)


@pytest.fixture(params=[secret_client, token_client])
def client(request):
    return request.param


def test_binary(client):
    with open(NORMAL_CSV, "rb") as file:
        data = file.read()
        res = client.put_binary(absolute_path=CSV_FILE, data=data)
        assert res is True, "Should upload file"
        binary = client.get_binary(CSV_FILE)
        assert binary == data, "Uploaded file should be same as file"


def test_csv(client):
    with open(NORMAL_CSV, encoding='utf-8') as file:
        spamreader = csv.reader(file)
        uploaded_csv = []
        for row in spamreader:
            uploaded_csv.append(row)
        res = client.put_csv(absolute_path=CSV_FILE, data=uploaded_csv)
        assert res is True, "Should upload csv"
        downloaded_csv = client.get_csv(absolute_path=CSV_FILE)
        assert type(downloaded_csv) is list, "csv should be a list"
        assert uploaded_csv == downloaded_csv, "Uploaded file should be same as file"


def test_pandas_csv(client):
    uploaded_csv = pd.read_csv(NORMAL_CSV)
    res = client.put_csv(absolute_path=CSV_FILE, data=uploaded_csv)
    assert res is True, "Should upload csv"
    downloaded_csv = client.get_pandas_csv(absolute_path=CSV_FILE)
    assert isinstance(downloaded_csv, pd.DataFrame), "should get data frame"
    assert uploaded_csv.equals(downloaded_csv), "Uploaded file should be same as file"


def test_json(client):
    with open(JSON_FILE) as file:
        spamreader = json_lines.reader(file)
        upload_json = []
        for row in spamreader:
            upload_json.append(row)
        res = client.put_json(absolute_path=JSON_FILE, data=upload_json)
        assert res is True, "Should upload json"
        downloaded_json = client.get_json(absolute_path=JSON_FILE)
        assert type(downloaded_json) is list, "json should be a list"
        assert upload_json == downloaded_json, "Uploaded file should be same as file"


def test_image(client):
    with open(IMAGE_FILE, "rb") as file:
        data = file.read()
        res = client.put_binary(absolute_path=IMAGE_FILE, data=data)
        assert res is True, "Should upload file"

        pimage = client.get_pil_image(IMAGE_FILE)
        assert isinstance(pimage, PngImageFile), "Should be PngImageFile"
        assert pimage.height is 256, "height should be 256"
        assert pimage.width is 256, "width should be 256"

        nimage = client.get_np_image(IMAGE_FILE)
        assert isinstance(nimage, np.ndarray), "Should be ndarray"
        assert nimage.shape[0] is 256, "height should be 256"
        assert nimage.shape[1] is 256, "width should be 256"


def test_list_file(client):
    file = client.list_file()
    assert type(file) is list, "file list should be a list"


def test_list_saved_query(client):
    saved_query = client.list_saved_query()
    assert isinstance(saved_query, list), "should get a list"


def test_use_saved_query(client):
    saved_query = client.list_saved_query()
    if len(saved_query) is 0:
        return
    query = client.use_saved_query(saved_query[0])
    query.wait_for_finish()
    print(query)
    assert isinstance(query, cannerflow.query.Query), "should get a query instance"
    assert type(query.id) == str, "query id should be a string"
    assert isinstance(query.status, SqlQueryStatus), "query status should a string"
    assert type(query.row_count) == int, "query row_count should be a int"
    assert type(query.statement_id) == str, "query statement_id should a string"

    assert isinstance(query.columns, list), (
        "columns should be list, but got" + query.columns
    )

    first = query.get_first()
    assert len(first) == 2, "should only get 2 (includs one column row)"

    all_data = query.get_all()
    assert (
        len(all_data) == query.row_count + 1
    ), "should get row_count + 1 (includes the column)"

    any_data = query.get(10, 3)
    assert len(any_data) <= 11, "should get less than 11 rows (includes the column)"

    query.data_format = "df"
    first = query.get_first()
    assert isinstance(first, pd.DataFrame), "should be a dataframe"

    query.data_format = "np"
    first = query.get_first()
    assert isinstance(first, np.ndarray), "should be a np array"


def test_gen_query_with_row_data(client):
    query = client.gen_query(
        "SELECT CAST(ROW(ARRAY[1], 2.0) AS ROW(x ARRAY(BIGINT), y DOUBLE))",
        cache_refresh=True,
    )
    query.wait_for_finish()
    assert query.status == SqlQueryStatus.FINISHED, "status must be finished"
    assert query.row_count == 1, "should have row_count"
    assert len(query.get_first(1)) == 2, "should get two columns"


def test_gen_query_with_empty_data(client):
    query = client.gen_query(
        "select * from tpch.tiny.lineitem limit 0", cache_refresh=True
    )
    query.wait_for_finish()
    assert query.status == SqlQueryStatus.FINISHED, "status must be finished"
    assert query.row_count == 0, "should have row_count"
    assert len(query.get_first(1)) == 1, "should only get column"


def test_gen_query_with_empty_data_in_df(client):
    query = client.gen_query(
        "select * from tpch.tiny.lineitem limit 0", cache_refresh=True, data_format="df"
    )
    query.wait_for_finish()
    assert query.status == SqlQueryStatus.FINISHED, "status must be finished"
    assert query.row_count == 0, "should have row_count"
    df = query.get_all()

    assert len(df.columns.tolist()) != 0, "should get columns in data frame"


def test_gen_query(client):
    query = client.gen_query(
        "select * from tpch.tiny.lineitem limit 1000", cache_refresh=True
    )
    query.wait_for_finish()
    assert query.status == SqlQueryStatus.FINISHED, "status must be finished"
    assert query.row_count > 0, "should have row_count"
    assert len(query.get_first(1)) == 2, "should get 2 rows include column"


def test_query_iterator(client):
    query = client.gen_query(
        "select * from tpch.tiny.lineitem limit 37435", cache_refresh=True
    )
    query.wait_for_finish()
    assert query.status == SqlQueryStatus.FINISHED, "status must be finished"
    assert query.row_count > 0, "should have row_count"
    count = 0
    for row in query:
        assert len(row) == 2, "should format as [columns, data]"
        count += 1
    assert count == 37435, "should get 37435 lines"


def test_query_iterator_small_data(client):
    query = client.gen_query(
        "select * from tpch.tiny.lineitem limit 100", cache_refresh=True
    )
    query.wait_for_finish()
    assert query.status == SqlQueryStatus.FINISHED, "status must be finished"
    assert query.row_count > 0, "should have row_count"
    count = 0
    for row in query:
        assert len(row) == 2, "should format as [columns, data]"
        count += 1
    assert count == 100, "should get 100 lines"


def test_query_iterator_df(client):
    query = client.gen_query(
        "select * from tpch.tiny.lineitem limit 100",
        cache_refresh=True,
        data_format="df",
    )
    query.wait_for_finish()
    assert query.status == SqlQueryStatus.FINISHED, "status must be finished"
    assert query.row_count > 0, "should have row_count"
    count = 0
    for row in query:
        assert isinstance(row, pd.DataFrame), "should be format as data frame"
        count += 1
    assert count == 100, "should get 100 lines"


def test_get_data_flow(client):
    query = client.gen_query(
        """ SELECT * FROM (
        VALUES (1, 10), (2, 20), (3, 30), (4, 40), (5, 50), (6, 60), (7, 70), (8, 80), (9, 90), (10, 100)
    ) AS testtable(col, col10)
    """
    )
    query.wait_for_finish(timeout=5, period=3)
    data: list = query.get_all()
    assert data[0] == ["col", "col10"]
    data = data[1:11]  # remove header
    assert len(data) is 10

    first3 = query.get_first(limit=3)
    first3 = first3[1:4]  # remove header
    assert len(first3) is 3
    assert first3 == data[0:3]

    last3 = query.get_last(limit=3)
    last3 = last3[1:4]  # remove header
    assert len(last3) is 3
    assert last3 == data[7:10]

    middle = query.get(limit=3, offset=3)
    middle = middle[1:4]  # remove header
    assert len(middle) is 3
    assert middle == data[3:6]


def test_show_nested_warning(client, caplog):
    nestedsql = """SELECT * FROM (
        VALUES (
            ARRAY[CAST(ROW(1, 2.0) AS ROW(x BIGINT, y DOUBLE)), CAST(ROW(2, 4.0) AS ROW(x BIGINT, y DOUBLE))],
            MAP(ARRAY['1', '2'], ARRAY[CAST(ROW(1, 2.0) AS ROW(x BIGINT, y DOUBLE)), CAST(ROW(2, 4.0) AS ROW(x BIGINT, y DOUBLE))]),
            CAST(ROW(1, CAST(ROW(1, 2.0) AS ROW(x BIGINT, y DOUBLE))) AS ROW(x BIGINT, y ROW(x BIGINT, y DOUBLE)))
        )
    ) AS nestedtable (arrayofrow, mapofrow, rowofrow)"""

    query = client.gen_query(nestedsql, data_format="df", fetch_by="storage")
    query.wait_for_finish()
    df = query.get_all()
    assert (
        len(caplog.messages) is 3
    ), f"Expect that got 3 warnings but {len(caplog.messages)}."
    assert caplog.messages[0].find(
        "arrayofrow"
    ), "Didn't get the warning for arrayofrow column."
    assert caplog.messages[1].find(
        "mapofrow"
    ), "Didn't get the warning for mapofrow column."
    assert caplog.messages[2].find(
        "rowofrow"
    ), "Didn't get the warning for rowofrow column."



from .operation_type import OperationType


class SqlQueryPayload(object):
    def __init__(self) -> None:
        self._name = OperationType.SQL_QUERY.value
        self._query = """
            query sqlQuery($where: SqlQueryWhereUniqueInput!) {
                sqlQuery(where: $where) {
                    id
                    status
                    error
                    location
                    statementId
                    rowCount
                    columns
                }
            }
        """

    def build(self, sql_query_id: str) -> dict:
        return {
            "operationName": self._name,
            "query": self._query,
            "variables": {"where": {"id": sql_query_id}},
        }



from .operation_type import OperationType


class CreateSqlQueryPayload(object):
    def __init__(self) -> None:
        self._name = OperationType.CREATE_SQL_QUERY.value
        self._query = """
            mutation createSqlQuery($data: SqlQueryCreateInput!, $where: SqlQueryWhereInput!) {
                createSqlQuery(data: $data, where: $where) {
                    id
                    status
                    error
                    location
                    statementId
                    rowCount
                    columns
                }
            }
        """

    def build(
        self, sql: str, workspace_id: str, cache_refresh: bool, cache_ttl: int
    ) -> dict:
        return {
            "operationName": self._name,
            "query": self._query,
            "variables": {
                "data": {
                    "sql": sql,
                    "cacheRefresh": cache_refresh,
                    "cacheTTL": cache_ttl,
                    "source": "NOTEBOOK",
                },
                "where": {"workspaceId": workspace_id},
            },
        }



import pandas as pd
from numpy import array
import time

__all__ = ["wait_until", "data_factory"]


def wait_until(somepredicate, timeout, period=1, *args, **kwargs):
    mustend = time.time() + timeout
    while time.time() < mustend:
        if somepredicate(*args, **kwargs):
            return True
        time.sleep(period)
    raise RuntimeError(
        f"Executing query exceed {timeout} seconds, please call query.wait_for_finish(timeout=int) to set a longer waiting time"
    )


def data_factory(data_format="list", **options):
    def get_df(columns, data):
        data_list = get_list(columns, data)
        # enforce to show the data frame dimesion e.g: [499 rows x 9 columns]
        pd.set_option("show_dimensions", True)
        return pd.DataFrame(data_list[1:], columns=data_list[0])

    def get_np(columns, data):
        return array(data)

    def get_list(columns, data, with_column=True):
        if with_column == True:
            column_names = list(map(lambda column: column["name"], columns))
            return [column_names] + data
        return data

    if data_format == "list":
        return get_list(**options)
    if data_format == "df":
        return get_df(**options)
    if data_format == "np":
        return get_np(**options)
    raise RuntimeError(f"Unexpected data_format {data_format}")



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import ast
import re
from setuptools import setup, find_packages
from os import path


_version_re = re.compile(r"__version__\s+=\s+(.*)")


with open("cannerflow/__init__.py", "rb") as f:
    version = str(
        ast.literal_eval(_version_re.search(f.read().decode("utf-8")).group(1))
    )

# read the contents of your README file
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "public_readme.md"), encoding="utf-8") as f:
    long_description = f.read()

tests_require = ["pytest", "pytest-runner"]


setup(
    name="cannerflow-python-client",
    author="Cannerflow Team",
    author_email="contact@canner.io",
    version=version,
    url="https://github.com/canner/cannerflow-python-client",
    packages=find_packages(),
    package_data={"": ["LICENSE", "README.md"]},
    description="Client for the Cannerflow",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="Apache 2.0",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX",
        "Operating System :: Microsoft :: Windows",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: Implementation :: CPython",
        "Topic :: Database :: Front-Ends",
    ],
    install_requires=[
        "requests",
        "pandas",
        "pillow",
        "aiohttp",
        "nest-asyncio",
        "pyarrow",
        "pillow",
        # numpy should >= 1.19.5 to let python install newer version depends on different python version) for supporting python3.7 - python 3.8.6 environment work 
        # solve numpy.ndarray size changed, may indicate binary incompatibility. Expected 88 from C header, got 80 from PyObject issue
        "numpy>=1.19.5",
        "fastparquet==0.4.1",
        "json-lines",
        "dataclasses==0.8;python_version >='3.6' and python_version <'3.7'"
    ],
    tests_requires=tests_require,
    python_requires='>=3.6.0, <=3.8.7'
)



from .create_sql_query import CreateSqlQueryPayload
from .sql_query import SqlQueryPayload
from .sql_result_pagination import SqlResultPaginationPayload
from .delete_statement import DeleteStatementPayload


__all__ = [
    "CreateSqlQueryPayload",
    "SqlQueryPayload",
    "SqlResultPaginationPayload",
    "DeleteStatementPayload",
]



from .csv_wrapper import CsvWrapper
from .image_wrapper import ImageWrapper
from .json_wrapper import JsonWrapper
from .file import File

__all__ = [
    "File",
    "CsvWrapper",
    "ImageWrapper",
    "JsonWrapper",
]